#import "BaseViewController.h"

@interface BaseViewController ()

@property(nonatomic, strong) UITextField *textField;
@property(nonatomic) CGRect rootViewOriginalFrame;

@end

@implementation BaseViewController

- (void)viewDidLoad {
    [super viewDidLoad];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillShow:)
                                                 name:UIKeyboardWillShowNotification
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];

    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)closeKeyboard {
    [self keyboardWillHide:nil];
}

- (void)keyboardWillShow:(NSNotification *)aNotification {
    UIView *activeView = [self findActiveResponderView:self.view];
    if (activeView == nil) {
        return;
    }

    [self slideUpKeyboardByNotification:aNotification andPartsFrame:activeView.frame];
}

- (void)keyboardWillHide:(NSNotification *)aNotification {
    [self slideDownKeyboardToOriginalPosition];
}

- (UIView *)findActiveResponderView:(UIView *)view {
    if ([view isFirstResponder]) {
        return view;

    } else {
        for (UIView *sub in view.subviews) {
            if ([sub isKindOfClass:[UIView class]]) {
                UIView *found = [self findActiveResponderView:sub];
                if (found != nil) {
                    return found;
                }
            }
        }
    }
    return nil;
}

- (CGFloat)slideUpKeyboardByNotification:(NSNotification *)notification
                           andPartsFrame:(CGRect)partsFrame {
    UIView *targetView = self.view;
    NSDictionary *dict = (NSDictionary *)notification.userInfo;
    if (dict != nil) {
        NSValue *s = (NSValue *)[dict valueForKey:UIKeyboardFrameEndUserInfoKey];
        CGRect keyboardSize = [s CGRectValue];

        CGFloat slidesize = partsFrame.origin.y + partsFrame.size.height +
                            [UIApplication sharedApplication].statusBarFrame.size.height +
                            self.navigationController.navigationBar.frame.size.height -
                            keyboardSize.origin.y;
        if (targetView.frame.origin.y != 0) {
            slidesize += targetView.frame.origin.y;
        }
        if (slidesize <= 0) {
            return 0.0;
        }

        self.rootViewOriginalFrame = targetView.frame;
        [UIView animateWithDuration:0.3
                         animations:^{
                           CGRect frame = targetView.frame;
                           frame.origin.y -= slidesize;
                           targetView.frame = frame;
                         }];
        return slidesize;
    }
    return 0.0;
}

- (void)slideDownKeyboardToOriginalPosition {
    if (self.rootViewOriginalFrame.size.height == 0) {
        return;
    }

    UIView *targetView = self.view;
    if (targetView.frame.size.height != self.rootViewOriginalFrame.size.height) {
        CGRect orvframe = self.rootViewOriginalFrame;
        orvframe.origin.y +=
            (self.rootViewOriginalFrame.size.height - targetView.frame.size.height);
        self.rootViewOriginalFrame = orvframe;
    }
    CGRect orvframe = self.rootViewOriginalFrame;
    orvframe.origin = CGPointZero;
    self.rootViewOriginalFrame = orvframe;
    [UIView animateWithDuration:0.3
                     animations:^{
                       targetView.frame = self.rootViewOriginalFrame;
                     }
                     completion:^(BOOL finished){
                     }];
}

@end
