#import "LogData.h"

@interface LogData ()

@property(nonatomic, strong) NSString *logText;

@end

@implementation LogData

- (id)init {
    self = [super init];
    if (self) {
        _logText = @"";
    }
    return self;
}

- (NSString *)toString {
    return self.logText;
}

- (void)clear {
    self.logText = @"";
}

- (void)addResponse:(NSArray<ResponseParamValue *> *)response {
    self.logText =
        [[LogData stringFromResponseArray:response] stringByAppendingString:self.logText];
}

+ (NSArray<ResponseParamValue *> *)sortedParams:
                                       (NSDictionary<MposResponseParam *, id> *)originalParams
                                    paramOrders:(NSArray<MposResponseParam *> *)paramOrders
                                 checkExistance:(BOOL)checkExistance {
    NSMutableDictionary<MposResponseParam *, id> *restParams =
        [originalParams isKindOfClass:[NSDictionary class]] ? [originalParams mutableCopy]
                                                            : [NSMutableDictionary dictionary];
    NSMutableArray<ResponseParamValue *> *resultParams = [NSMutableArray array];

    [paramOrders
        enumerateObjectsUsingBlock:^(MposResponseParam *param, NSUInteger idx, BOOL *stop) {
          if (restParams[param] != nil) {
              ResponseParamValue *item =
                  [[ResponseParamValue alloc] initWithParam:param
                                                      value:restParams[param]
                                                     status:ResponseParamValueStatusOk
                                                   isBraces:NO];
              [restParams removeObjectForKey:param];
              [resultParams addObject:item];
          } else {
              if (!checkExistance) {
                  return;
              }
              ResponseParamValue *item =
                  [[ResponseParamValue alloc] initWithParam:param
                                                      value:@""
                                                     status:ResponseParamValueStatusMissing
                                                   isBraces:NO];
              [resultParams addObject:item];
          }
        }];

    [restParams
        enumerateKeysAndObjectsUsingBlock:^(MposResponseParam *param, id value, BOOL *stop) {
          ResponseParamValue *item =
              [[ResponseParamValue alloc] initWithParam:param
                                                  value:value
                                                 status:ResponseParamValueStatusUnused
                                               isBraces:NO];
          [resultParams addObject:item];
        }];

    return resultParams;
}

+ (NSArray<ResponseParamValue *> *)sortedPaymentParams:
    (NSDictionary<MposResponseParam *, id> *)response {
    NSArray<MposResponseParam *> *orders = @[
        MposResponseParam.status,
        MposResponseParam.errorCode,
        MposResponseParam.errorMessage,
        MposResponseParam.resultCode,
        MposResponseParam.orderId,
        MposResponseParam.orderReferenceId,
        MposResponseParam.paymentMethodCode,
        MposResponseParam.paymentSource,
        MposResponseParam.amount,
        MposResponseParam.discountAmount,
        MposResponseParam.totalAmount,
        MposResponseParam.paymentTime,
        MposResponseParam.orderDescription,
        MposResponseParam.cardNumber,
        MposResponseParam.cardNumberToken,
        MposResponseParam.cardProviderCode,
        MposResponseParam.cardExpiry,
        MposResponseParam.cardType,
        MposResponseParam.jpo,
        MposResponseParam.authCode,
        MposResponseParam.acquirerCode,
        MposResponseParam.centerAcquirerId,
        MposResponseParam.centerTradeId,
        MposResponseParam.centerProcessId,
        MposResponseParam.centerReferenceCode,
        MposResponseParam.centerTransactionType,
        MposResponseParam.centerTransactionDatetime,
        MposResponseParam.centerTerminalId,
        MposResponseParam.centerMerchantId,
        MposResponseParam.centerMerchantName,
        MposResponseParam.cupRequestTime,
        MposResponseParam.settledAmount,
        MposResponseParam.settledScale,
        MposResponseParam.settledCurrency,
        MposResponseParam.emvAppId,
        MposResponseParam.emvAtc,
        MposResponseParam.emvAppLabel,
        MposResponseParam.panSeqNumber,
        MposResponseParam.aosa,
        MposResponseParam.cvmType,
        MposResponseParam.cvmr,
        MposResponseParam.tvr,
        MposResponseParam.tsi,
        MposResponseParam.txnCryptogram,
        MposResponseParam.mileageMembershipNo,
        MposResponseParam.usageType,
        MposResponseParam.lastUseDate,
    ];

    BOOL checkExistance = [response[MposResponseParam.status] isEqualToString:@"success"];
    return [self sortedParams:response paramOrders:orders checkExistance:checkExistance];
}

+ (NSArray<ResponseParamValue *> *)sortedSearchParams:
    (NSDictionary<MposResponseParam *, id> *)response {
    NSArray<MposResponseParam *> *orders = @[
        MposResponseParam.status,
        MposResponseParam.errorCode,
        MposResponseParam.errorMessage,
        MposResponseParam.resultCode,
        MposResponseParam.historyList,
    ];

    NSArray<MposResponseParam *> *historyOrders = @[
        MposResponseParam.orderId,
        MposResponseParam.orderReferenceId,
        MposResponseParam.paymentMethodCategory,
        MposResponseParam.paymentMethodCode,
        MposResponseParam.paymentSource,
        MposResponseParam.orderStatus,
        MposResponseParam.amount,
        MposResponseParam.discountAmount,
        MposResponseParam.totalAmount,
        MposResponseParam.registeredTime,
        MposResponseParam.paymentTime,
        MposResponseParam.status,
        MposResponseParam.resultCode,
        MposResponseParam.orderDescription,
        MposResponseParam.cardNumber,
        MposResponseParam.cardNumberToken,
        MposResponseParam.cardProviderCode,
        MposResponseParam.cardExpiry,
        MposResponseParam.cardType,
        MposResponseParam.jpo,
        MposResponseParam.authCode,
        MposResponseParam.acquirerCode,
        MposResponseParam.centerAcquirerId,
        MposResponseParam.centerTradeId,
        MposResponseParam.centerProcessId,
        MposResponseParam.centerReferenceCode,
        MposResponseParam.centerTransactionType,
        MposResponseParam.centerTransactionDatetime,
        MposResponseParam.centerRefundDatetime,
        MposResponseParam.centerTerminalId,
        MposResponseParam.centerMerchantId,
        MposResponseParam.centerMerchantName,
        MposResponseParam.cupRequestTime,
        MposResponseParam.settledAmount,
        MposResponseParam.settledScale,
        MposResponseParam.settledCurrency,
        MposResponseParam.emvAppId,
        MposResponseParam.emvAtc,
        MposResponseParam.emvAppLabel,
        MposResponseParam.panSeqNumber,
        MposResponseParam.aosa,
        MposResponseParam.cvmType,
        MposResponseParam.cvmr,
        MposResponseParam.tvr,
        MposResponseParam.tsi,
        MposResponseParam.txnCryptogram,
        MposResponseParam.mileageMembershipNo,
        MposResponseParam.usageType,
        MposResponseParam.lastUseDate,
    ];

    BOOL checkExistance = [response[MposResponseParam.status] isEqualToString:@"success"];
    NSArray *rootParams = [self sortedParams:response
                                 paramOrders:orders
                              checkExistance:checkExistance];
    for (ResponseParamValue *rootParam in rootParams) {
        if (rootParam.param == MposResponseParam.historyList) {
            NSArray *originalHistoryPages = (NSArray *)rootParam.value;
            NSMutableArray *sortedHistoryPages = [NSMutableArray array];
            for (NSDictionary *historyParams in originalHistoryPages) {
                NSArray *sortedHistoryParams = [self sortedParams:historyParams
                                                      paramOrders:historyOrders
                                                   checkExistance:checkExistance];
                [sortedHistoryPages addObject:sortedHistoryParams];
            }
            rootParam.value = sortedHistoryPages;
            break;
        }
    }
    return rootParams;
}

+ (NSArray<ResponseParamValue *> *)sortedMailParams:
    (NSDictionary<MposResponseParam *, id> *)response {
    NSArray<MposResponseParam *> *orders = @[
        MposResponseParam.status,
        MposResponseParam.errorCode,
        MposResponseParam.errorMessage,
        MposResponseParam.resultCode,
    ];

    BOOL checkExistance = [response[MposResponseParam.status] isEqualToString:@"success"];
    return [self sortedParams:response paramOrders:orders checkExistance:checkExistance];
}

+ (NSArray<ResponseParamValue *> *)sortedRefundParams:
    (NSDictionary<MposResponseParam *, id> *)response {
    NSArray<MposResponseParam *> *orders = @[
        MposResponseParam.status,
        MposResponseParam.errorCode,
        MposResponseParam.errorMessage,
        MposResponseParam.resultCode,
        MposResponseParam.orderId,
        MposResponseParam.paymentMethodCode,
        MposResponseParam.paymentSource,
        MposResponseParam.amount,
        MposResponseParam.discountAmount,
        MposResponseParam.totalAmount,
        MposResponseParam.cardNumber,
        MposResponseParam.cardNumberToken,
        MposResponseParam.cardProviderCode,
        MposResponseParam.cardExpiry,
        MposResponseParam.cardType,
        MposResponseParam.jpo,
        MposResponseParam.authCode,
        MposResponseParam.acquirerCode,
        MposResponseParam.centerAcquirerId,
        MposResponseParam.centerTradeId,
        MposResponseParam.centerProcessId,
        MposResponseParam.centerReferenceCode,
        MposResponseParam.centerTransactionType,
        MposResponseParam.centerTransactionDatetime,
        MposResponseParam.centerRefundDatetime,
        MposResponseParam.centerTerminalId,
        MposResponseParam.centerMerchantId,
        MposResponseParam.centerMerchantName,
        MposResponseParam.cupRequestTime,
        MposResponseParam.emvAppId,
        MposResponseParam.emvAtc,
        MposResponseParam.emvAppLabel,
        MposResponseParam.panSeqNumber,
        MposResponseParam.aosa,
        MposResponseParam.cvmType,
        MposResponseParam.cvmr,
        MposResponseParam.tvr,
        MposResponseParam.tsi,
        MposResponseParam.txnCryptogram,
        MposResponseParam.mileageMembershipNo,
    ];

    BOOL checkExistance = [response[MposResponseParam.status] isEqualToString:@"success"];
    return [self sortedParams:response paramOrders:orders checkExistance:checkExistance];
}

+ (NSArray<ResponseParamValue *> *)sortedCancelParams:
    (NSDictionary<MposResponseParam *, id> *)response {
    NSArray<MposResponseParam *> *orders = @[
        MposResponseParam.status,
        MposResponseParam.errorCode,
        MposResponseParam.errorMessage,
        MposResponseParam.resultCode,
        MposResponseParam.orderId,
        MposResponseParam.paymentMethodCode,
        MposResponseParam.jpo,
        MposResponseParam.cardNumber,
        MposResponseParam.cardProviderCode,
        MposResponseParam.cardExpiry,
        MposResponseParam.authCode,
        MposResponseParam.acquirerCode,
    ];

    BOOL checkExistance = [response[MposResponseParam.status] isEqualToString:@"success"];
    return [self sortedParams:response paramOrders:orders checkExistance:checkExistance];
}

+ (NSArray<ResponseParamValue *> *)sortedPasswordChangeParams:
    (NSDictionary<MposResponseParam *, id> *)response {
    NSArray<MposResponseParam *> *orders = @[
        MposResponseParam.status,
        MposResponseParam.errorCode,
        MposResponseParam.errorMessage,
        MposResponseParam.resultCode,
    ];

    BOOL checkExistance = [response[MposResponseParam.status] isEqualToString:@"success"];
    return [self sortedParams:response paramOrders:orders checkExistance:checkExistance];
}

+ (NSArray<ResponseParamValue *> *)sortedReportParams:
    (NSDictionary<MposResponseParam *, id> *)response {
    NSArray<MposResponseParam *> *orders = @[
        MposResponseParam.status,
        MposResponseParam.errorCode,
        MposResponseParam.errorMessage,
        MposResponseParam.resultCode,
        MposResponseParam.reportTimeFrom,
        MposResponseParam.reportTimeTo,
        MposResponseParam.paymentMethodCode,
        MposResponseParam.orderStatus,
        MposResponseParam.salesReport,
        MposResponseParam.mileageReport,
    ];

    NSArray<MposResponseParam *> *salesOrders = @[
        MposResponseParam.captureTotalAmount,
        MposResponseParam.refundTotalAmount,
        MposResponseParam.differenceTotalAmount,
        MposResponseParam.captureCount,
        MposResponseParam.refundCount,
        MposResponseParam.differenceCount,
    ];

    NSArray<MposResponseParam *> *mileageOrders = @[
        MposResponseParam.accrualTotalAmount,
        MposResponseParam.cancelTotalAmount,
        MposResponseParam.differenceTotalAmount,
        MposResponseParam.accrualCount,
        MposResponseParam.cancelCount,
        MposResponseParam.differenceCount,
    ];

    BOOL checkExistance = [response[MposResponseParam.status] isEqualToString:@"success"];
    NSArray *rootParams = [self sortedParams:response
                                 paramOrders:orders
                              checkExistance:checkExistance];
    for (ResponseParamValue *rootParam in rootParams) {
        if (rootParam.param == MposResponseParam.salesReport) {
            rootParam.value = [self sortedParams:rootParam.value
                                     paramOrders:salesOrders
                                  checkExistance:checkExistance];
            rootParam.isBraces = YES;
            break;
        }
    }
    for (ResponseParamValue *rootParam in rootParams) {
        if (rootParam.param == MposResponseParam.mileageReport) {
            rootParam.value = [self sortedParams:rootParam.value
                                     paramOrders:mileageOrders
                                  checkExistance:checkExistance];
            rootParam.isBraces = YES;
            break;
        }
    }
    return rootParams;
}

+ (NSArray<ResponseParamValue *> *)sortedUpdateBTParams:
    (NSDictionary<MposResponseParam *, id> *)response {
    NSArray<MposResponseParam *> *orders = @[
        MposResponseParam.status,
        MposResponseParam.errorCode,
        MposResponseParam.errorMessage,
        MposResponseParam.resultCode,
    ];

    BOOL checkExistance = [response[MposResponseParam.status] isEqualToString:@"success"];
    return [self sortedParams:response paramOrders:orders checkExistance:checkExistance];
}

+ (NSString *)stringFromResponseObject:(ResponseParamValue *)obj {
    return [self stringFromResponseElement:obj];
}

+ (NSString *)stringFromResponseArray:(NSArray<ResponseParamValue *> *)array {
    NSString *text = @"";
    for (id item in array) {
        text = [text stringByAppendingString:[self stringFromResponseElement:item]];
    }
    return text;
}

+ (NSString *)stringFromResponseElement:(id)obj {
    if ([obj isKindOfClass:NSString.class]) {
        return obj;
    }

    if ([obj isKindOfClass:ResponseParamValue.class]) {
        ResponseParamValue *item = obj;
        NSString *text = @"";
        text = [text stringByAppendingString:item.param.rawValue];
        text = [text stringByAppendingString:@": "];

        if (item.isBraces) {
            text = [text stringByAppendingString:@"{"];
        }

        text = [text stringByAppendingString:[self stringFromResponseElement:item.value]];
        if (![item.value isKindOfClass:NSArray.class]) {
            text = [text stringByAppendingString:@"\n"];
        }

        if (item.isBraces) {
            text = [text stringByAppendingString:@"}\n"];
        }

        return text;
    }

    if ([obj isKindOfClass:NSArray.class]) {
        if ([obj count] == 0) {
            return @"";
        }

        if ([obj[0] isKindOfClass:ResponseParamValue.class]) {
            NSString *text = @"";
            for (id item in obj) {
                text = [text stringByAppendingString:[self stringFromResponseElement:item]];
            }
            return text;

        } else {
            NSString *text = @"[\n";
            BOOL isFirst = YES;
            for (id item in obj) {
                if (!isFirst) {
                    text = [text stringByAppendingString:@", "];
                }

                text = [text stringByAppendingString:[self stringFromResponseElement:item]];
                isFirst = NO;
            }
            text = [text stringByAppendingString:@"]\n"];
            return text;
        }
    }

    return @"";
}

@end
