#import "MailViewController.h"
#import "LogViewController.h"
#import "MposHandler+VTsdkSampleApp.h"
#import "SettingData.h"
#import "TextFieldController.h"
#import "UIAlertController+VTsdkSampleApp.h"

@interface MailViewController ()

@property(nonatomic, strong) LogData *logData;
@property(weak, nonatomic) IBOutlet UISwitch *orderIdSwitch;
@property(weak, nonatomic) IBOutlet UITextField *orderIdTextField;
@property(weak, nonatomic) IBOutlet UISwitch *emailSwitch;
@property(weak, nonatomic) IBOutlet UITextField *emailTextField;
@property(weak, nonatomic) IBOutlet UISwitch *mailTypeSwitch;
@property(weak, nonatomic) IBOutlet UITextField *mailTypeTextField;
@property(weak, nonatomic) IBOutlet UIView *coverView;
@property(strong, nonatomic) TextFieldController *textFieldController;
@property(nullable, strong, nonatomic) MposHandler *handler;

@end

@implementation MailViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.logData = [[LogData alloc] init];
    self.navigationItem.rightBarButtonItem =
        [UIBarButtonItem logButtonItemWithTarget:self action:@selector(onLogButton:)];

    self.textFieldController =
        [[TextFieldController alloc] initWithOwnerViewController:self
                                                       didChange:^(NSString *response){
                                                       }];
    self.orderIdTextField.delegate = self.textFieldController;
    self.emailTextField.delegate = self.textFieldController;
    self.mailTypeTextField.delegate = self.textFieldController;
    [self.coverView setHidden:YES];
    self.handler = [[MposHandler alloc] init];
}

- (void)onLogButton:(id)sender {
    [self.navigationController
        pushViewController:[LogViewController logViewControllerWithOwnerViewController:self
                                                                               logData:self.logData]
                  animated:YES];
}

- (IBAction)onSendButton:(id)sender {
    NSMutableDictionary<MposRequestParam *, NSString *> *requestParam =
        [NSMutableDictionary dictionary];

    if (self.orderIdSwitch.on) {
        [requestParam setObject:self.orderIdTextField.text forKey:MposRequestParam.orderId];
    }

    if (self.emailSwitch.on) {
        [requestParam setObject:self.emailTextField.text forKey:MposRequestParam.email];
    }

    if (self.mailTypeSwitch.on) {
        [requestParam setObject:self.mailTypeTextField.text forKey:MposRequestParam.mailType];
    }

    __weak typeof(self) _self = self;

    [self.coverView setHidden:NO];
    [self.handler requestSendReceipt:[SettingData requestHeader]
                       requestParams:requestParam
                            complete:^(NSDictionary<MposResponseParam *, id> *param) {
                              [self.coverView setHidden:YES];
                              NSArray *sortedParams = [LogData sortedMailParams:param];
                              [_self.logData addResponse:sortedParams];
                              UIAlertController *alertController =
                                  [UIAlertController alertControllerWithResponse:sortedParams];
                              [_self presentViewController:alertController
                                                  animated:YES
                                                completion:^{
                                                }];
                            }];
}

@end
