#import "PasswordViewController.h"
#import "LogData.h"
#import "LogViewController.h"
#import "MposHandler+VTsdkSampleApp.h"
#import "SettingData.h"
#import "TextFieldController.h"
#import "UIAlertController+VTsdkSampleApp.h"

@interface PasswordViewController ()

@property(nonatomic, strong) LogData *logData;
@property(weak, nonatomic) IBOutlet UISwitch *oldPasswordSwitch;
@property(weak, nonatomic) IBOutlet UITextField *oldPasswordTextField;
@property(weak, nonatomic) IBOutlet UISwitch *nextPasswordSwitch;
@property(weak, nonatomic) IBOutlet UITextField *nextPasswordTextField;
@property(weak, nonatomic) IBOutlet UIView *coverView;
@property(strong, nonatomic) TextFieldController *textFieldController;
@property(nullable, strong, nonatomic) MposHandler *handler;

@end

@implementation PasswordViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.logData = [[LogData alloc] init];
    self.navigationItem.rightBarButtonItem =
        [UIBarButtonItem logButtonItemWithTarget:self action:@selector(onLogButton:)];

    self.textFieldController =
        [[TextFieldController alloc] initWithOwnerViewController:self
                                                       didChange:^(NSString *response){
                                                       }];
    self.oldPasswordTextField.delegate = self.textFieldController;
    self.nextPasswordTextField.delegate = self.textFieldController;
    [self.coverView setHidden:YES];
    self.handler = [[MposHandler alloc] init];
}

- (void)onLogButton:(id)sender {
    [self.navigationController
        pushViewController:[LogViewController logViewControllerWithOwnerViewController:self
                                                                               logData:self.logData]
                  animated:YES];
}

- (IBAction)onChangeButton:(id)sender {
    NSMutableDictionary<MposRequestParam *, NSString *> *requestParam =
        [NSMutableDictionary dictionary];

    if (self.oldPasswordSwitch.on) {
        [requestParam setObject:self.oldPasswordTextField.text forKey:MposRequestParam.oldPasswd];
    }

    if (self.nextPasswordSwitch.on) {
        [requestParam setObject:self.nextPasswordTextField.text forKey:MposRequestParam.newPasswd];
    }

    __weak typeof(self) _self = self;

    [self.coverView setHidden:NO];
    [self.handler requestChangePassword:[SettingData requestHeader]
                          requestParams:requestParam
                               complete:^(NSDictionary<MposResponseParam *, id> *param) {
                                 [self.coverView setHidden:YES];
                                 NSArray *sortedParams = [LogData sortedPasswordChangeParams:param];
                                 [_self.logData addResponse:sortedParams];
                               }];
}

@end
