#import "PaymentViewController.h"
#import "LogViewController.h"
#import "MposHandler+VTsdkSampleApp.h"
#import "SettingData.h"
#import "TextFieldController.h"
#import "UIAlertController+VTsdkSampleApp.h"
#import "UIBarButtonItem+VTsdkSampleApp.h"

@interface PaymentViewController ()

@property(strong, nonatomic) LogData *logData;
@property(weak, nonatomic) IBOutlet UITextField *amountTextField;
@property(weak, nonatomic) IBOutlet UITextField *descriptionTextField;
@property(weak, nonatomic) IBOutlet UISwitch *descriptionSwitch;
@property(weak, nonatomic) IBOutlet UISwitch *displayInputAmountPageSwitch;
@property(weak, nonatomic) IBOutlet UITextField *displayInputAmountPageTextField;
@property(weak, nonatomic) IBOutlet UISwitch *displayCompletePageSwitch;
@property(weak, nonatomic) IBOutlet UITextField *displayCompletePageTextField;
@property(nullable, strong, nonatomic) TextFieldController *textFieldController;
@property(weak, nonatomic) IBOutlet UISwitch *paymentMethodCodeSwitch;
@property(weak, nonatomic) IBOutlet UITextField *paymentMethodCodeTextField;
@property(nullable, strong, nonatomic) MposHandler *handler;

@end

@implementation PaymentViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.logData = [[LogData alloc] init];
    self.navigationItem.rightBarButtonItem =
        [UIBarButtonItem logButtonItemWithTarget:self action:@selector(onLogButton:)];

    self.textFieldController =
        [[TextFieldController alloc] initWithOwnerViewController:self
                                                       didChange:^(NSString *response){
                                                       }];
    self.amountTextField.delegate = self.textFieldController;
    self.descriptionTextField.delegate = self.textFieldController;
    self.displayInputAmountPageTextField.delegate = self.textFieldController;
    self.displayCompletePageTextField.delegate = self.textFieldController;
    self.paymentMethodCodeTextField.delegate = self.textFieldController;
    self.handler = [[MposHandler alloc] init];
}

- (void)onLogButton:(id)sender {
    [self.navigationController
        pushViewController:[LogViewController logViewControllerWithOwnerViewController:self
                                                                               logData:self.logData]
                  animated:YES];
}

- (IBAction)onPayButton:(id)sender {
    NSMutableDictionary<MposRequestParam *, NSString *> *requestParam =
        [NSMutableDictionary dictionary];

    [requestParam setObject:self.amountTextField.text forKey:MposRequestParam.amount];

    if (self.descriptionSwitch.on) {
        [requestParam setObject:self.descriptionTextField.text
                         forKey:MposRequestParam.orderDescription];
    }

    if (self.displayInputAmountPageSwitch.on) {
        [requestParam setObject:self.displayInputAmountPageTextField.text
                         forKey:MposRequestParam.inputAmountView];
    }

    if (self.displayCompletePageSwitch.on) {
        [requestParam setObject:self.displayCompletePageTextField.text
                         forKey:MposRequestParam.completeView];
    }

    if (self.paymentMethodCodeSwitch.on) {
        [requestParam setObject:self.paymentMethodCodeTextField.text
                         forKey:MposRequestParam.paymentMethodCode];
    }

    __weak typeof(self) _self = self;

    [self.handler startPayment:[SettingData requestHeader]
                 requestParams:requestParam
                      complete:^(NSDictionary<MposResponseParam *, id> *param) {
                        NSArray *sortedParams = [LogData sortedPaymentParams:param];
                        [_self.logData addResponse:sortedParams];
                        UIAlertController *alertController =
                            [UIAlertController alertControllerWithResponse:sortedParams];
                        [_self presentViewController:alertController
                                            animated:YES
                                          completion:^{
                                          }];
                      }];
}

@end
