#import "ReaderAppUpdateBTViewController.h"
#import "LogData.h"
#import "LogViewController.h"
#import "MposHandler+VTsdkSampleApp.h"
#import "SettingData.h"
#import "TextFieldController.h"
#import "UIAlertController+VTsdkSampleApp.h"

@interface ReaderAppUpdateBTViewController ()

@property(nonatomic, strong) LogData *logData;
@property(weak, nonatomic) IBOutlet UISwitch *progressDialogSwitch;
@property(weak, nonatomic) IBOutlet UITextField *progressDialogTextField;
@property(weak, nonatomic) IBOutlet UISwitch *btConnectTimeoutSwitch;
@property(weak, nonatomic) IBOutlet UITextField *btConnectTimeoutTextField;
@property(weak, nonatomic) IBOutlet UIView *coverView;
@property(strong, nonatomic) TextFieldController *textFieldController;
@property(nullable, strong, nonatomic) MposHandler *handler;

@end

@implementation ReaderAppUpdateBTViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.logData = [[LogData alloc] init];
    self.navigationItem.rightBarButtonItem =
        [UIBarButtonItem logButtonItemWithTarget:self action:@selector(onLogButton:)];
    self.textFieldController =
        [[TextFieldController alloc] initWithOwnerViewController:self
                                                       didChange:^(NSString *response){
                                                       }];
    self.progressDialogTextField.delegate = self.textFieldController;
    self.btConnectTimeoutTextField.delegate = self.textFieldController;
    [self.coverView setHidden:YES];
    self.handler = [[MposHandler alloc] init];
}

- (void)onLogButton:(id)sender {
    [self.navigationController
        pushViewController:[LogViewController logViewControllerWithOwnerViewController:self
                                                                               logData:self.logData]
                  animated:YES];
}

- (IBAction)onStartButton:(id)sender {
    NSMutableDictionary<MposRequestParam *, NSString *> *requestParam =
        [NSMutableDictionary dictionary];

    if (self.progressDialogSwitch.on) {
        [requestParam setObject:self.progressDialogTextField.text
                         forKey:MposRequestParam.progressDialog];
    }

    if (self.btConnectTimeoutSwitch.on) {
        [requestParam setObject:self.btConnectTimeoutTextField.text
                         forKey:MposRequestParam.btConnectTimeout];
    }

    __weak typeof(self) _self = self;

    [self.coverView setHidden:NO];
    [self.handler requestReaderAppUpdateBT:[SettingData requestHeader]
                             requestParams:requestParam
                                  complete:^(NSDictionary<MposResponseParam *, id> *param) {
                                    [self.coverView setHidden:YES];
                                    NSArray *sortedParams = [LogData sortedUpdateBTParams:param];
                                    [_self.logData addResponse:sortedParams];
                                    UIAlertController *alertController = [UIAlertController
                                        alertControllerWithResponse:sortedParams];
                                    [_self presentViewController:alertController
                                                        animated:YES
                                                      completion:^{
                                                      }];
                                  }];
}

@end
