#import "ReportViewController.h"
#import "LogData.h"
#import "LogViewController.h"
#import "MposHandler+VTsdkSampleApp.h"
#import "SettingData.h"
#import "TextFieldController.h"
#import "UIAlertController+VTsdkSampleApp.h"

@interface ReportViewController ()

@property(nonatomic, strong) LogData *logData;
@property(weak, nonatomic) IBOutlet UISwitch *reportTimeFromSwitch;
@property(weak, nonatomic) IBOutlet UITextField *reportTimeFromTextField;
@property(weak, nonatomic) IBOutlet UISwitch *reportTimeToSwitch;
@property(weak, nonatomic) IBOutlet UITextField *reportTimeToTextField;
@property(weak, nonatomic) IBOutlet UISwitch *paymentMethodCodeSwitch;
@property(weak, nonatomic) IBOutlet UITextField *paymentMethodCodeTextField;
@property(weak, nonatomic) IBOutlet UISwitch *orderStatusSwitch;
@property(weak, nonatomic) IBOutlet UITextField *orderStatusTextField;
@property(weak, nonatomic) IBOutlet UIView *coverView;
@property(strong, nonatomic) TextFieldController *textFieldController;
@property(nullable, strong, nonatomic) MposHandler *handler;

@end

@implementation ReportViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.logData = [[LogData alloc] init];
    self.navigationItem.rightBarButtonItem =
        [UIBarButtonItem logButtonItemWithTarget:self action:@selector(onLogButton:)];
    self.textFieldController =
        [[TextFieldController alloc] initWithOwnerViewController:self
                                                       didChange:^(NSString *response){
                                                       }];
    self.reportTimeFromTextField.delegate = self.textFieldController;
    self.reportTimeToTextField.delegate = self.textFieldController;
    self.paymentMethodCodeTextField.delegate = self.textFieldController;
    self.orderStatusTextField.delegate = self.textFieldController;
    [self.coverView setHidden:YES];
    self.handler = [[MposHandler alloc] init];
}

- (void)onLogButton:(id)sender {
    [self.navigationController
        pushViewController:[LogViewController logViewControllerWithOwnerViewController:self
                                                                               logData:self.logData]
                  animated:YES];
}

- (IBAction)onStartButton:(id)sender {
    NSMutableDictionary<MposRequestParam *, NSString *> *requestParam =
        [NSMutableDictionary dictionary];

    if (self.reportTimeFromSwitch.on) {
        [requestParam setObject:self.reportTimeFromTextField.text
                         forKey:MposRequestParam.reportTimeFrom];
    }

    if (self.reportTimeToSwitch.on) {
        [requestParam setObject:self.reportTimeToTextField.text
                         forKey:MposRequestParam.reportTimeTo];
    }
    if (self.paymentMethodCodeSwitch.on) {
        [requestParam setObject:self.paymentMethodCodeTextField.text
                         forKey:MposRequestParam.paymentMethodCode];
    }

    if (self.orderStatusSwitch.on) {
        [requestParam setObject:self.orderStatusTextField.text forKey:MposRequestParam.orderStatus];
    }

    __weak typeof(self) _self = self;

    [self.coverView setHidden:NO];
    [self.handler requestReport:[SettingData requestHeader]
                  requestParams:requestParam
                       complete:^(NSDictionary<MposResponseParam *, id> *param) {
                         [self.coverView setHidden:YES];
                         NSArray *sortedParams = [LogData sortedReportParams:param];
                         [_self.logData addResponse:sortedParams];
                         UIAlertController *alertController =
                             [UIAlertController alertControllerWithResponse:sortedParams];
                         [_self presentViewController:alertController
                                             animated:YES
                                           completion:^{
                                           }];
                       }];
}

@end
