#import "SearchResultViewController.h"
#import "SearchCell.h"

@implementation SearchResultViewController {
    NSArray<NSArray<ResponseParamValue *> *> *histories;
}

- (void)setParams:(NSArray<ResponseParamValue *> *)params {
    for (ResponseParamValue *data in params) {
        if (data.param == MposResponseParam.historyList) {
            histories = (NSArray<NSArray<ResponseParamValue *> *> *)data.value;
            break;
        }
    }
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return histories.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    SearchCell *cell = [tableView dequeueReusableCellWithIdentifier:@"SearchCell"];
    NSArray<ResponseParamValue *> *items = histories[indexPath.row];

    cell.pageLabel.text = [NSString stringWithFormat:@"%ld", (indexPath.row + 1)];

    for (SearchCellItemView *itemView in cell.itemViews) {
        itemView.hidden = YES;
        itemView.titleLabel.text = @"";
        itemView.valueLabel.text = @"";
    }

    UIColor *normalColor = [UIColor colorWithRed:85 / 255.0
                                           green:85 / 255.0
                                            blue:85 / 255.0
                                           alpha:1.0];
    [items enumerateObjectsUsingBlock:^(ResponseParamValue *item, NSUInteger idx, BOOL *stop) {
      SearchCellItemView *itemView = cell.itemViews[idx];
      itemView.hidden = NO;
      itemView.titleLabel.text = item.param.rawValue;
      itemView.valueLabel.text = (item.value != nil) ? item.value : @"";

      UIColor *color;
      switch (item.status) {
          case ResponseParamValueStatusOk:
              color = normalColor;
              break;
          case ResponseParamValueStatusMissing:
              color = UIColor.brownColor;
              break;
          case ResponseParamValueStatusUnused:
              color = UIColor.blueColor;
              break;
      }
      itemView.titleLabel.textColor = color;
      itemView.valueLabel.textColor = color;
    }];

    return cell;
}

@end

@implementation SearchResultViewController (Factory)

+ (instancetype)
    searchResultViewControllerWithOwnerViewController:(UIViewController *)ownerViewController
                                               params:(NSArray<ResponseParamValue *> *)params {
    SearchResultViewController *viewController = [[ownerViewController storyboard]
        instantiateViewControllerWithIdentifier:NSStringFromClass([self class])];
    viewController.params = params;
    return viewController;
}

@end
