#import "SearchViewController.h"
#import "LogViewController.h"
#import "MposHandler+VTsdkSampleApp.h"
#import "SearchResultViewController.h"
#import "SettingData.h"
#import "TextFieldController.h"
#import "UIAlertController+VTsdkSampleApp.h"

@interface SearchViewController ()

@property(nonatomic, strong) LogData *logData;
@property(weak, nonatomic) IBOutlet UITextField *transactionCountTextField;
@property(weak, nonatomic) IBOutlet UISwitch *orderReferenceIdSwitch;
@property(weak, nonatomic) IBOutlet UITextField *orderReferenceIdTextField;
@property(weak, nonatomic) IBOutlet UISwitch *orderIdSwitch;
@property(weak, nonatomic) IBOutlet UITextField *orderIdTextField;
@property(weak, nonatomic) IBOutlet UISwitch *registeredTimeFromSwitch;
@property(weak, nonatomic) IBOutlet UITextField *registeredTimeFromTextField;
@property(weak, nonatomic) IBOutlet UISwitch *registeredTimeToSwitch;
@property(weak, nonatomic) IBOutlet UITextField *registeredTimeToTextField;
@property(weak, nonatomic) IBOutlet UIView *coverView;
@property(nullable, strong, nonatomic) TextFieldController *textFieldController;
@property(nullable, strong, nonatomic) MposHandler *handler;

@end

@implementation SearchViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.logData = [[LogData alloc] init];
    self.navigationItem.rightBarButtonItem =
        [UIBarButtonItem logButtonItemWithTarget:self action:@selector(onLogButton:)];
    self.textFieldController =
        [[TextFieldController alloc] initWithOwnerViewController:self
                                                       didChange:^(NSString *response){
                                                       }];
    self.orderReferenceIdTextField.delegate = self.textFieldController;
    self.transactionCountTextField.delegate = self.textFieldController;
    self.orderIdTextField.delegate = self.textFieldController;
    self.registeredTimeFromTextField.delegate = self.textFieldController;
    self.registeredTimeToTextField.delegate = self.textFieldController;
    [self.coverView setHidden:YES];
    self.handler = [[MposHandler alloc] init];
}

- (void)onLogButton:(id)sender {
    [self.navigationController
        pushViewController:[LogViewController logViewControllerWithOwnerViewController:self
                                                                               logData:self.logData]
                  animated:YES];
}

- (IBAction)onSearchButton:(id)sender {
    NSMutableDictionary<MposRequestParam *, NSString *> *requestParam =
        [NSMutableDictionary dictionary];

    [requestParam setObject:self.transactionCountTextField.text
                     forKey:MposRequestParam.transactionCount];

    if (self.orderReferenceIdSwitch.on) {
        [requestParam setObject:self.orderReferenceIdTextField.text
                         forKey:MposRequestParam.orderReferenceId];
    }

    if (self.orderIdSwitch.on) {
        [requestParam setObject:self.orderIdTextField.text forKey:MposRequestParam.orderId];
    }

    if (self.registeredTimeFromSwitch.on) {
        [requestParam setObject:self.registeredTimeFromTextField.text
                         forKey:MposRequestParam.registeredTimeFrom];
    }

    if (self.registeredTimeToSwitch.on) {
        [requestParam setObject:self.registeredTimeToTextField.text
                         forKey:MposRequestParam.registeredTimeTo];
    }

    __weak typeof(self) _self = self;

    [self.coverView setHidden:NO];
    [self.handler
        requestSearchTransactionHistory:[SettingData requestHeader]
                          requestParams:requestParam
                               complete:^(NSDictionary<MposResponseParam *, id> *param) {
                                 [self.coverView setHidden:YES];

                                 NSArray<ResponseParamValue *> *sortedParams =
                                     [LogData sortedSearchParams:param];
                                 NSString *status = param[MposResponseParam.status];

                                 if ([status isEqualToString:@"failure"]) {
                                     [_self presentViewController:
                                                [UIAlertController
                                                    alertControllerWithResponse:sortedParams]
                                                         animated:YES
                                                       completion:^{
                                                       }];
                                 } else {
                                     [_self.logData addResponse:sortedParams];
                                     [_self.navigationController
                                         pushViewController:
                                             [SearchResultViewController
                                                 searchResultViewControllerWithOwnerViewController:
                                                     _self
                                                                                            params:
                                                                                                sortedParams]
                                                   animated:YES];
                                 }
                               }];
}

@end
