#import "SettingData.h"

static NSString *const SettingDataSaveDataKey = @"SettingData";

@implementation SettingData

- (id)init {
    self = [super init];
    if (self) {
        _userID = @"";
        _userPassword = @"";
        _merchantPassword = @"";
        _readerType = @"iPOS";
        _appNameType = @"sdksample";
        _appVersion = @"1.0";
        _locale = @"ja";
        _cardThreeWay = @"";
        _cameraFacing = @"";
        _isUserPassword = [NSNumber numberWithBool:YES];
        _isMerchantPassword = [NSNumber numberWithBool:YES];
        _isReaderType = [NSNumber numberWithBool:YES];
        _isCardThreeWay = [NSNumber numberWithBool:YES];
        _isCameraFacing = [NSNumber numberWithBool:YES];
    }
    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder {
    [coder encodeObject:self.userID forKey:@"userID"];
    [coder encodeObject:self.userPassword forKey:@"userPassword"];
    [coder encodeObject:self.merchantPassword forKey:@"merchantPassword"];
    [coder encodeObject:self.readerType forKey:@"readerType"];
    [coder encodeObject:self.appNameType forKey:@"appNameType"];
    [coder encodeObject:self.appVersion forKey:@"appVersion"];
    [coder encodeObject:self.locale forKey:@"locale"];
    [coder encodeObject:self.cardThreeWay forKey:@"cardThreeWay"];
    [coder encodeObject:self.cameraFacing forKey:@"cameraFacing"];
    [coder encodeObject:self.isUserPassword forKey:@"isUserPassword"];
    [coder encodeObject:self.isMerchantPassword forKey:@"isMerchantPassword"];
    [coder encodeObject:self.isReaderType forKey:@"isReaderType"];
    [coder encodeObject:self.isCardThreeWay forKey:@"isCardThreeWay"];
    [coder encodeObject:self.isCameraFacing forKey:@"isCameraFacing"];
}

- (id)initWithCoder:(NSCoder *)coder {
    self = [super init];
    if (self) {
        _userID = [coder decodeObjectForKey:@"userID"];
        _userPassword = [coder decodeObjectForKey:@"userPassword"];
        _merchantPassword = [coder decodeObjectForKey:@"merchantPassword"];
        _readerType = [coder decodeObjectForKey:@"readerType"];
        _appNameType = [coder decodeObjectForKey:@"appNameType"];
        _appVersion = [coder decodeObjectForKey:@"appVersion"];
        _locale = [coder decodeObjectForKey:@"locale"];
        _cardThreeWay = [coder decodeObjectForKey:@"cardThreeWay"];
        _cameraFacing = [coder decodeObjectForKey:@"cameraFacing"];
        _isUserPassword = [coder decodeObjectForKey:@"isUserPassword"];
        _isMerchantPassword = [coder decodeObjectForKey:@"isMerchantPassword"];
        _isReaderType = [coder decodeObjectForKey:@"isReaderType"];
        _isCardThreeWay = [coder decodeObjectForKey:@"isCardThreeWay"];
        _isCameraFacing = [coder decodeObjectForKey:@"isCameraFacing"];
    }
    return self;
}

+ (instancetype)settingData {
    id tempObj = [[NSUserDefaults standardUserDefaults] objectForKey:SettingDataSaveDataKey];
    SettingData *obj = nil;

    if (tempObj) {
        obj = [NSKeyedUnarchiver unarchiveObjectWithData:tempObj];

    } else {
        obj = [[[self class] alloc] init];
        [obj save];
    }
    return obj;
}

- (void)save {
    [[NSUserDefaults standardUserDefaults]
        setObject:[NSKeyedArchiver archivedDataWithRootObject:self]
           forKey:SettingDataSaveDataKey];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

@end

@implementation SettingData (MposHandler)

+ (NSDictionary *)requestHeader {
    SettingData *settingData = [self settingData];

    NSMutableDictionary *headerParam = [NSMutableDictionary dictionary];
    [headerParam setObject:settingData.userID forKey:MposRequestParam.userId];

    if ([settingData.isUserPassword boolValue]) {
        [headerParam setObject:settingData.userPassword forKey:MposRequestParam.userPassword];
    } else if ([settingData.isMerchantPassword boolValue]) {
        [headerParam setObject:settingData.merchantPassword
                        forKey:MposRequestParam.merchantPassword];
    }

    if ([settingData.isReaderType boolValue]) {
        [headerParam setObject:settingData.readerType forKey:MposRequestParam.readerType];
    }

    [headerParam setObject:settingData.locale forKey:MposRequestParam.locale];

    if ([settingData.isCardThreeWay boolValue]) {
        [headerParam setObject:settingData.cardThreeWay forKey:MposRequestParam.cardThreeWay];
    }

    if ([settingData.isCameraFacing boolValue]) {
        [headerParam setObject:settingData.cameraFacing forKey:MposRequestParam.cameraFacing];
    }
    return headerParam;
}

@end
