#import "SettingViewController.h"
#import "SettingData.h"
#import "TextFieldController.h"

@interface SettingViewController ()

@property(weak, nonatomic) IBOutlet UITextField *userIdTextField;
@property(weak, nonatomic) IBOutlet UISwitch *userPasswordSwitch;
@property(weak, nonatomic) IBOutlet UITextField *userPasswordTextField;
@property(weak, nonatomic) IBOutlet UISwitch *merchantPasswordSwitch;
@property(weak, nonatomic) IBOutlet UITextField *merchantPasswordTextField;
@property(weak, nonatomic) IBOutlet UISwitch *readerTypeSwitch;
@property(weak, nonatomic) IBOutlet UITextField *readerTypeTextField;
@property(weak, nonatomic) IBOutlet UITextField *appNameTextField;
@property(weak, nonatomic) IBOutlet UITextField *appVersionTextField;
@property(weak, nonatomic) IBOutlet UITextField *localTextField;
@property(weak, nonatomic) IBOutlet UISwitch *cardThreeWaySwitch;
@property(weak, nonatomic) IBOutlet UITextField *cardThreeWayTextField;
@property(weak, nonatomic) IBOutlet UISwitch *cameraFacingSwitch;
@property(weak, nonatomic) IBOutlet UITextField *cameraFacingTextField;
@property(strong, nonatomic) TextFieldController *textFieldController;

@end

@implementation SettingViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.navigationItem.rightBarButtonItem =
        [UIBarButtonItem saveButtonItemWithTarget:self action:@selector(onSaveButton:)];

    self.textFieldController =
        [[TextFieldController alloc] initWithOwnerViewController:self
                                                       didChange:^(NSString *response){
                                                       }];
    self.userIdTextField.delegate = self.textFieldController;
    self.userPasswordTextField.delegate = self.textFieldController;
    self.merchantPasswordTextField.delegate = self.textFieldController;
    self.readerTypeTextField.delegate = self.textFieldController;
    self.appNameTextField.delegate = self.textFieldController;
    self.appVersionTextField.delegate = self.textFieldController;
    self.localTextField.delegate = self.textFieldController;
    self.cardThreeWayTextField.delegate = self.textFieldController;
    self.cameraFacingTextField.delegate = self.textFieldController;

    SettingData *settingData = [SettingData settingData];
    self.userIdTextField.text = settingData.userID;
    self.userPasswordTextField.text = settingData.userPassword;
    self.merchantPasswordTextField.text = settingData.merchantPassword;
    self.readerTypeTextField.text = settingData.readerType;
    self.appNameTextField.text = settingData.appNameType;
    self.appVersionTextField.text = settingData.appVersion;
    self.localTextField.text = settingData.locale;
    self.cardThreeWayTextField.text = settingData.cardThreeWay;
    self.cameraFacingTextField.text = settingData.cameraFacing;
    self.userPasswordSwitch.on = settingData.isUserPassword.boolValue;
    self.merchantPasswordSwitch.on = settingData.isMerchantPassword.boolValue;
    self.readerTypeSwitch.on = settingData.isReaderType.boolValue;
    self.cardThreeWaySwitch.on = settingData.isCardThreeWay.boolValue;
    self.cameraFacingSwitch.on = settingData.isCameraFacing.boolValue;
}

- (void)viewWillDisappear:(BOOL)animated {
    [self doSave];

    [super viewWillDisappear:animated];
}

- (void)onSaveButton:(id)sender {
    [self doSave];

    [self.navigationController popViewControllerAnimated:YES];
}

- (void)doSave {
    SettingData *settingData = [SettingData settingData];
    settingData.userID = self.userIdTextField.text;
    settingData.userPassword = self.userPasswordTextField.text;
    settingData.merchantPassword = self.merchantPasswordTextField.text;
    settingData.readerType = self.readerTypeTextField.text;
    settingData.appNameType = self.appNameTextField.text;
    settingData.appVersion = self.appVersionTextField.text;
    settingData.locale = self.localTextField.text;
    settingData.cardThreeWay = self.cardThreeWayTextField.text;
    settingData.cameraFacing = self.cameraFacingTextField.text;
    settingData.isUserPassword = [NSNumber numberWithBool:self.userPasswordSwitch.on];
    settingData.isMerchantPassword = [NSNumber numberWithBool:self.merchantPasswordSwitch.on];
    settingData.isReaderType = [NSNumber numberWithBool:self.readerTypeSwitch.on];
    settingData.isCardThreeWay = [NSNumber numberWithBool:self.cardThreeWaySwitch.on];
    settingData.isCameraFacing = [NSNumber numberWithBool:self.cameraFacingSwitch.on];
    [settingData save];
}

@end
