#import "TextFieldController.h"

@interface TextFieldController ()

@property(nonatomic, weak) UIViewController *ownerViewController;
@property(nonatomic, copy) void (^didChange)(NSString *respons);
@property(nonatomic, strong) UITapGestureRecognizer *tapGesture;
@property(nonatomic, strong) UITextField *textField;

@end

@implementation TextFieldController

- (instancetype)initWithOwnerViewController:(UIViewController *)ownerViewController
                                  didChange:(void (^)(NSString *response))didChange {
    self = [super init];

    if (self) {
        _ownerViewController = ownerViewController;
        _didChange = didChange;
        _tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self
                                                              action:@selector(didTapInTextField:)];
        _tapGesture.numberOfTapsRequired = 1;
    }
    return self;
}

- (void)textFieldEditingChanged:(UITextField *)textField {
    if (self.didChange) {
        self.didChange(textField.text);
    }
}

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField {
    self.textField = textField;
    [self.ownerViewController.view addGestureRecognizer:self.tapGesture];
    return YES;
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
    [self.ownerViewController.view removeGestureRecognizer:self.tapGesture];
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    if (textField.returnKeyType != UIReturnKeyNext) {
        [textField resignFirstResponder];
    }
    return YES;
}

- (void)didTapInTextField:(id)sender {
    [self.textField resignFirstResponder];
}

@end
